@@ -52,14 +52,14 @@ trivfs_S_io_reauthenticate (struct trivfs_protid *cred,
   assert (newright != MACH_PORT_NULL);
 
   err = iohelp_reauth (&newcred->user, auth, rendport, newright, 1);
-  mach_port_deallocate (mach_task_self (), rendport);
+  if (!err)
+    mach_port_deallocate (mach_task_self (), rendport);
   mach_port_deallocate (mach_task_self (), auth);
   if (err)
     return err;
 
   mach_port_deallocate (mach_task_self (), newright);
-  if (idvec_contains (newcred->user->uids, 0))
-    newcred->isroot = 1;
+  newcred->isroot = _is_privileged (newcred->user->uids);
 
   newcred->hook = cred->hook;
   newcred->po = cred->po;
