@@ -30,11 +30,14 @@
 
 static mach_port_t real_defpager, dev_master;
 
+/* Our port class.  */
+struct port_class *trivfs_protid_class;
+
 static error_t
 allowed (mach_port_t port, int mode)
 {
-  struct trivfs_protid *cred = ports_lookup_port
-    (0, port, trivfs_protid_portclasses[0]);
+  struct trivfs_protid *cred
+    = ports_lookup_port (0, port, trivfs_protid_class);
   if (!cred)
     return MIG_BAD_ID;
   error_t result = (cred->po->openmodes & mode) ? 0 : EACCES;
@@ -60,6 +63,19 @@ S_default_pager_info (mach_port_t default_pager, default_pager_info_t *info)
     ?: default_pager_info (real_defpager, info);
 }
 
+kern_return_t
+S_default_pager_storage_info (mach_port_t default_pager,
+			      vm_size_array_t *size,
+			      mach_msg_type_number_t *sizeCnt,
+			      vm_size_array_t *free,
+			      mach_msg_type_number_t *freeCnt,
+			      data_t *name,
+			      mach_msg_type_number_t *nameCnt)
+{
+  return allowed (default_pager, O_READ)
+    ?: default_pager_storage_info (real_defpager, size, sizeCnt, free, freeCnt, name, nameCnt);
+}
+
 kern_return_t
 S_default_pager_objects (mach_port_t default_pager,
 			 default_pager_object_array_t *objects,
@@ -233,10 +249,16 @@ int
 proxy_defpager_demuxer (mach_msg_header_t *inp,
 			mach_msg_header_t *outp)
 {
-  extern int default_pager_server (mach_msg_header_t *, mach_msg_header_t *);
-
-  return default_pager_server (inp, outp)
-    || trivfs_demuxer (inp, outp);
+  mig_routine_t routine;
+  if ((routine = default_pager_server_routine (inp)) ||
+      (routine = NULL, trivfs_demuxer (inp, outp)))
+    {
+      if (routine)
+        (*routine) (inp, outp);
+      return TRUE;
+    }
+  else
+    return FALSE;
 }
 
 int
@@ -264,10 +286,12 @@ main (int argc, char **argv)
   if (bootstrap == MACH_PORT_NULL)
     error (1, 0, "Must be started as a translator");
 
-  trivfs_fsid = getpid ();
+  err = trivfs_add_protid_port_class (&trivfs_protid_class);
+  if (err)
+    error (1, 0, "error creating protid port class");
 
   /* Reply to our parent.  */
-  err = trivfs_startup (bootstrap, 0, 0, 0, 0, 0, &fsys);
+  err = trivfs_startup (bootstrap, 0, 0, 0, trivfs_protid_class, 0, &fsys);
   mach_port_deallocate (mach_task_self (), bootstrap);
   if (err)
     error (4, err, "Contacting parent");
